﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.DataImporter.Data;
using HeuristicLab.DataImporter.Command.View;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;


namespace HeuristicLab.DataImporter.Command {
  [StorableClass]
  [ViewableCommandInfoAttribute("Interpolate Missing Values", 1, ColumnGroupState.Sorted | ColumnGroupState.DoubleColumnSelected, "Time Series",
Position = 2)]
  public class InterpolateMissingValues : FillMissingValueCommandBase {
    private DateTimeColumn columnToSample;

    private InterpolateMissingValues()
      : base(null, string.Empty, null) {
    }

    public InterpolateMissingValues(DataSet dataSet, string columnGroupName, int[] affectedColumns)
      : base(dataSet, columnGroupName, affectedColumns) {
    }

    public override string Description {
      get { return "Interpolate Missing Values"; }
    }

    public override void Execute() {
      base.Execute();
      if (!(ColumnGroup.GetColumn(ColumnGroup.SortedColumnIndexes.ElementAt(0)) is DateTimeColumn)
   || ColumnGroup.SortedColumnIndexes.Count != 1)
        throw new CommandExecutionException("ColumnGroup must be sorted after a datetime column.", this);
      columnToSample = (DateTimeColumn)ColumnGroup.Columns.ElementAt(ColumnGroup.SortedColumnIndexes.ElementAt(0));
      if (columnToSample.SortOrder != SortOrder.Ascending)
        throw new CommandExecutionException("ColumnGroup must be sorted ascending by the datetime column.", this);
      if (columnToSample.ContainsNullValues)
        throw new CommandExecutionException("The datetime column must not contain null values.", this);

      oldSortOrder = ColumnGroup.SortOrdersForColumns.ToList();
      oldSortedColumnIndexes = new List<int>(ColumnGroup.SortedColumnIndexes);
      DoubleColumn column;
      int startIndex = -1;
      int endIndex = -1;
      foreach (int col in AffectedColumns) {
        if (ColumnGroup.GetColumn(col) is DoubleColumn && ColumnGroup.GetColumn(col).ContainsNullValues) {
          column = (DoubleColumn)ColumnGroup.Columns.ElementAt(col);
          nullValues.Add(col, new List<int>());
          startIndex = FindNextNotNullValueIndex(column, startIndex);
          endIndex = FindNextNotNullValueIndex(column, startIndex);
          while (endIndex != -1) {
            if (endIndex - startIndex != 1) {
              nullValues[col].AddRange(Enumerable.Range(startIndex + 1, endIndex - startIndex - 1));
              InterpolateValues(column, startIndex, endIndex);
            }
            startIndex = endIndex;
            endIndex = FindNextNotNullValueIndex(column, endIndex);
          }
          startIndex = -1;
        }
      }
      ColumnGroup.FireChanged();
      columnToSample = null;
      this.ColumnGroup = null;
    }

    private int FindNextNotNullValueIndex(DoubleColumn column, int startIndex) {
      int endIndex = -1;
      for (int i = startIndex + 1; i < column.TotalValuesCount && endIndex == -1; i++)
        if (column.GetValue(i) != null)
          endIndex = i;
      return endIndex;
    }

    private void InterpolateValues(DoubleColumn column, int startIndex, int endIndex) {
      double startValue = (double)column.GetValue(startIndex);
      double endValue = (double)column.GetValue(endIndex);
      DateTime start = (DateTime)columnToSample.GetValue(startIndex);
      DateTime end = (DateTime)columnToSample.GetValue(endIndex);

      for (int i = startIndex + 1; i < endIndex; i++) {
        if (column.GetValue(i) != null)
          throw new CommandExecutionException("Tried to manipulate non null value.", this);
        column.ChangeValue(i, ((double)(((DateTime)columnToSample.GetValue(i) - start).TotalSeconds / (end - start).TotalSeconds)) *
          (endValue - startValue) + startValue);
      }
    }
  }
}
